@@ -28,8 +28,8 @@
 HELANAL --- analysis of protein helices
 =======================================
 
-:Author:  Benjamin Hall <benjamin.a.hall@ucl.ac.uk>, Oliver Beckstein
-:Year:    2009, 2011
+:Author:  Benjamin Hall <benjamin.a.hall@ucl.ac.uk>, Oliver Beckstein, Xavier Deupi
+:Year:    2009, 2011, 2013
 :License: GNU General Public License v2 (or higher)
 
 The :mod:`MDAnalysis.analysis.helanal` module is a Python implementation of the
@@ -56,8 +56,8 @@ structures.
 
 .. _HELANAL: http://www.ccrnp.ncifcrf.gov/users/kumarsan/HELANAL/helanal.html
 .. _Helanal webserver: http://nucleix.mbu.iisc.ernet.in/helanal/helanal.shtml
-.. _`helanal.f`: http://www.webcitation.org/5y1RpVJtF
-.. helanal.f: http://www.ccrnp.ncifcrf.gov/users/kumarsan//HELANAL/helanal.f
+.. `helanal.f`: http://www.webcitation.org/5y1RpVJtF
+.. _`helanal.f`: http://www.ccrnp.ncifcrf.gov/users/kumarsan/HELANAL/helanal.f
 
 Background
 ----------
@@ -68,7 +68,7 @@ HELANAL_ can be used to characterize the geometry of helices with a minimum 9
 residues. The geometry of an alpha helix is characterized by computing local
 helix axes and local helix origins for four contiguous C-Alpha atoms, using the
 procedure of Sugeta and Miyazawa [Sugeta1967]_ and sliding this window over
-the length of the helix in steps of one C-Alpha atom.
+the length of the helix in steps of one C-alpha atom.
 
 The angles between successive local helix axes can identify *local bends* or
 *kinks* as well as occurrence of *smooth curvature* in the helix. A matrix, whose
@@ -112,7 +112,6 @@ Functions
 
 """
 
-from __future__ import with_statement
 
 import sys
 import os
@@ -130,6 +129,8 @@ except ImportError:
         def deg2rad(x):             # no need for the numpy out=[] argument
                 return x*numpy.pi/180.0
 
+import MDAnalysis
+
 def center(coordinates):
         """Return the geometric center (centroid) of the coordinates.
 
@@ -191,6 +192,7 @@ def helanal_trajectory(universe, selection="name CA", start=None, end=None, begi
                        matrix_filename="bending_matrix.dat", origin_pdbfile="origin.pdb",
                        summary_filename="summary.txt", screw_filename="screw.xvg",
                        tilt_filename="local_tilt.xvg", fitted_tilt_filename="fit_tilt.xvg",
+                       bend_filename="local_bend.xvg",twist_filename="unit_twist.xvg",
                        prefix="helanal_", ref_axis=None):
         """Perform HELANAL_ helix analysis on all frames in *universe*.
 
@@ -228,6 +230,12 @@ def helanal_trajectory(universe, selection="name CA", start=None, end=None, begi
            *tilt_filename*
               Output file- tilt of line of best fit applied to origin axes
               ["local_tilt.xvg"]
+           *bend_filename*
+              Output file- local bend angles between successive local helix axes
+              ["local_bend.xvg"]
+           *twist_filename*
+              Output file- local unit twist between successive helix turns
+              ["unit_twist.xvg"]
            *prefix*
               Prefix to add to all output file names; set to ``None`` to disable
               ["helanal__"]
@@ -242,7 +250,7 @@ def helanal_trajectory(universe, selection="name CA", start=None, end=None, begi
                 # two atoms
                 ref_axis = numpy.asarray(ref_axis)
 
-        if start is None and stop is None:
+        if start is None and end is None:
                 pass
         else:
                 if start is None:
@@ -269,12 +277,16 @@ def helanal_trajectory(universe, selection="name CA", start=None, end=None, begi
                 screw_filename = prefix + screw_filename
                 tilt_filename = prefix + tilt_filename
                 fitted_tilt_filename = prefix + fitted_tilt_filename
+                bend_filename = prefix + bend_filename
+                twist_filename = prefix + twist_filename
         backup_file(matrix_filename)
         backup_file(origin_pdbfile)
         backup_file(summary_filename)
         backup_file(screw_filename)
         backup_file(tilt_filename)
         backup_file(fitted_tilt_filename)
+        backup_file(bend_filename)
+        backup_file(twist_filename)
 
         global_height = []
         global_twist = []
@@ -319,7 +331,20 @@ def helanal_trajectory(universe, selection="name CA", start=None, end=None, begi
                 #global_screw.append(local_screw_angles)
                 global_fitted_tilts.append(rad2deg(fit_tilt))
 
+
                 #print out rotations across the helix to a file
+                with open(twist_filename,"a") as twist_output:
+                        print >> twist_output, frame,
+                        for loc_twist in twist:
+                                print >> twist_output, loc_twist,
+                        print >> twist_output, ""
+
+                with open(bend_filename,"a") as bend_output:
+                        print >> bend_output, frame,
+                        for loc_bend in bending_angles:
+                                print >> bend_output, loc_bend,
+                        print >> bend_output, ""
+
                 with open(screw_filename,"a") as rot_output:
                         print >> rot_output, frame,
                         for rotation in local_screw_angles:
@@ -348,6 +373,7 @@ def helanal_trajectory(universe, selection="name CA", start=None, end=None, begi
                 formated_frame ="%10d" % frame
 
                 print '\r',formated_time,' ps' , formated_frame,
+
                 sys.stdout.flush()
 
         print '\nComplete'
@@ -473,7 +499,7 @@ def helanal_main(pdbfile, selection="name CA", start=None, end=None, ref_axis=No
         """
 
         universe = MDAnalysis.Universe(pdbfile, permissive=permissive)
-        if start is None and stop is None:
+        if start is None and end is None:
                 pass
         else:
                 if start is None:
@@ -502,7 +528,7 @@ def helanal_main(pdbfile, selection="name CA", start=None, end=None, ref_axis=No
         #calculate local bending matrix(now it is looking at all i, j combinations)
         for i in local_helix_axes:
                 for j in local_helix_axes:
-                        if i == j:
+                        if (i == j).all():
                                 angle = 0.
                         else:
                                 angle = rad2deg(numpy.arccos(vecscaler(i,j)))
@@ -536,6 +562,11 @@ def helanal_main(pdbfile, selection="name CA", start=None, end=None, ref_axis=No
                 output = "%8.1f\t" % angle
                 print output,
         print ''
+        print "Unit twist angles:"
+        for twist_ang in twist:
+                outputtwist = "%8.1f\t" % twist_ang
+                print outputtwist,
+        print ''
 
         #calculate best fit vector and tilt of said vector
         fit_vector, fit_tilt = vector_of_best_fit(origins)
