@@ -47,9 +47,10 @@ for the XTC and TRR format.
 
 """
 
-import os.path
+import os
 import errno
 import numpy
+import sys
 
 import libxdrfile, statno
 from MDAnalysis.coordinates import base
@@ -129,7 +130,7 @@ class TrjWriter(base.Writer):
     #: override to define trajectory format of the reader (XTC or TRR)
     format = None
 
-    def __init__(self, filename, numatoms, start=0, step=1, delta=1.0, precision=1000.0, remarks=None,
+    def __init__(self, filename, numatoms, start=0, step=1, delta=None, precision=1000.0, remarks=None,
                   convert_units=None):
         ''' Create a new TrjWriter
 
@@ -141,11 +142,14 @@ class TrjWriter(base.Writer):
 
         :Keywords:
           *start*
-             starting timestep
+             starting timestep; only used when *delta* is set.
           *step*
-             skip between subsequent timesteps
+             skip between subsequent timesteps; only used when *delta* is set.
           *delta*
-             timestep
+             timestep to use. If set will override any time information contained in the
+             passed :class:`Timestep` objects; otherwise that will be used. If in the 
+             latter case :attr:`Timestep.time` is unavailable the TrjWriter will default
+             to setting the trajectory time at 1 MDAnalysis unit (typically 1ps) per step.
           *precision*
              accuracy for lossy XTC format [1000]
           *convert_units*
@@ -158,6 +162,13 @@ class TrjWriter(base.Writer):
         if numatoms == 0:
             raise ValueError("TrjWriter: no atoms in output trajectory")
         self.filename = filename
+        # Convert filename to ascii because of SWIG bug.
+        # See: http://sourceforge.net/p/swig/feature-requests/75
+        # Only needed for Python < 3
+        if sys.version_info[0] < 3:
+            if isinstance(filename, unicode):
+                self.filename = filename.encode("UTF-8")
+
         if convert_units is None:
             convert_units = MDAnalysis.core.flags['convert_gromacs_lengths']
         self.convert_units = convert_units    # convert length and time to base units on the fly?
@@ -169,7 +180,7 @@ class TrjWriter(base.Writer):
         self.delta = delta
         self.remarks = remarks
         self.precision = precision  # only for XTC
-        self.xdrfile = libxdrfile.xdrfile_open(filename, 'w')
+        self.xdrfile = libxdrfile.xdrfile_open(self.filename, 'w')
 
         self.ts = None
 
@@ -200,6 +211,19 @@ class TrjWriter(base.Writer):
         """Generic writer for XTC and TRR with minimum intelligence; override if necessary."""
 
         # (1) data common to XTC and TRR
+
+        # Time-writing logic: if the writer was created with a delta parameter,
+        #  use delta*(start+step*frames_written)
+        #  otherwise use the provided Timestep obj time attribute. Fallback to 1 if not present.
+        if self.delta is None:
+            try:
+                time = ts.time
+            except AttributeError:
+                # Default to 1 time unit.
+                time = self.start+self.step*self.frames_written
+        else:
+            time = (self.start+self.step*self.frames_written)*self.delta
+
         if self.convert_units:
             # make a copy of the scaled positions so that the in-memory
             # timestep is not changed (would have lead to wrong results if
@@ -208,16 +232,9 @@ class TrjWriter(base.Writer):
             # very big systems because we temporarily create a new array pos
             # for each frame written
             pos = self.convert_pos_to_native(ts._pos, inplace=False)
-            try:
-                time = self.convert_time_to_native(ts.time, inplace=False)
-            except AttributeError:
-                time = ts.frame * self.convert_time_to_native(self.delta, inplace=False)
+            time = self.convert_time_to_native(time, inplace=False)
         else:
             pos = ts._pos
-            try:
-                time = ts.time
-            except AttributeError:
-                time = ts.frame * self.delta
         if not hasattr(ts, 'step'):
             # bogus, should be actual MD step number, i.e. frame * delta/dt
             ts.step = ts.frame
@@ -225,7 +242,7 @@ class TrjWriter(base.Writer):
 
         # (2) have to treat XTC and TRR somewhat differently
         if self.format == 'XTC':
-            status = libxdrfile.write_xtc(self.xdrfile, ts.step, time, unitcell, pos, self.precision)
+            status = libxdrfile.write_xtc(self.xdrfile, int(ts.step), float(time), unitcell, pos, self.precision)
         elif self.format == 'TRR':
             if not hasattr(ts, 'lmbda'):
                 ts.lmbda = 1.0
@@ -238,10 +255,17 @@ class TrjWriter(base.Writer):
                 # 0-velocities (no need to convert those); add it to ts as a sideeffect
                 # so that we don't have to reallocate next time
                 velocities = ts._velocities = numpy.zeros((3,ts.numatoms), dtype=numpy.float32)
-            if not hasattr(ts, '_forces'):
-                ts._forces = numpy.zeros((3,ts.numatoms), dtype=numpy.float32)
-            status = libxdrfile.write_trr(self.xdrfile, ts.step, time, ts.lmbda, unitcell,
-                                          pos, velocities, ts._forces)
+            if hasattr(ts, '_forces'):
+                if self.convert_units:
+                    forces = self.convert_forces_to_native(ts._forces, inplace=False)
+                else:
+                    forces = ts._forces
+            else:
+                # 0-forces (no need to convert those); add it to ts as a sideeffect
+                # so that we don't have to reallocate next time
+                forces = ts._forces = numpy.zeros((3,ts.numatoms), dtype=numpy.float32)
+            status = libxdrfile.write_trr(self.xdrfile, int(ts.step), float(time), float(ts.lmbda), unitcell,
+                                          pos, velocities, forces)
         else:
             raise NotImplementedError("Gromacs trajectory format %s not known." % self.format)
         return status
@@ -278,38 +302,111 @@ class TrjReader(base.Reader):
     #: writer class that matches this reader (override appropriately)
     _Writer = TrjWriter
 
-    def __init__(self, filename, convert_units=None, **kwargs):
+    def __init__(self, filename, convert_units=None, sub=None, **kwargs):
+        """
+        :Arguments:
+            *filename*
+                the name of the trr file.
+            *sub*
+
+                an numpy integer array of what subset of trajectory atoms to load into
+                the timestep. Intended to work similarly to the 'sub' argument to gromac's trjconv.
+
+                This is usefull when one has a Universe loaded with only an unsolvated protein, and
+                wants to read a solvated trajectory.
+
+                The length of this array must be <= to the actual number of atoms in the trajectory, and
+                equal to number of atoms in the Universe.
+
+        """
         self.filename = filename
+        # Convert filename to ascii because of SWIG bug.
+        # See: http://sourceforge.net/p/swig/feature-requests/75
+        # Only needed for Python < 3
+        if sys.version_info[0] < 3:
+            if isinstance(filename, unicode):
+                self.filename = filename.encode("UTF-8")
+
+        # Check if file is not too big file for a 32-bit.
+        # underlying xdrfile is indeed limited by the max integer value
+        if os.path.getsize(filename) > sys.maxint: # This test works even on a 32-bit system
+            raise OSError("Filesize is too big to be handled by xdrfile library (filesize: %.1fMB - max size: %.1fMB)"
+                          % (os.path.getsize(filename)/1.e6, sys.maxint/1.e6))
+
         if convert_units is None:
             convert_units = MDAnalysis.core.flags['convert_gromacs_lengths']
         self.convert_units = convert_units  # convert length and time to base units on the fly?
         self.xdrfile = None
-        self.__numatoms = None
+
         self.__numframes = None  # takes a long time, avoid accessing self.numframes
         self.skip_timestep = 1   # always 1 for xdr files
         self.__delta = None      # compute from time in first two frames!
         self.fixed = 0           # not relevant for Gromacs xtc/trr
+        self.__offsets = None    # storage of offsets in the file
         self.skip = 1
         self.periodic = False
+
+        # actual number of atoms in the trr file
+        # first time file is opened, exception should be thrown if bad file
+        self.__trr_numatoms = self._read_trj_natoms(self.filename)
+
+        # logic for handling sub sections of trr:
+        # this class has some tmp buffers into which the libxdrfile functions read the
+        # entire frame, then this class copies the relevant sub section into the timestep.
+        # the sub section logic is contained entierly within this class.
+
+        # check to make sure sub is valid (if given)
+        if sub is not None:
+            # only valid type
+            if not isinstance(sub, numpy.ndarray) or len(sub.shape) != 1 or sub.dtype.kind != 'i':
+                raise TypeError("sub MUST be a single dimensional numpy array of integers")
+            if len(sub) > self.__trr_numatoms:
+                raise ValueError("sub MUST be less than or equal to the number of actual trr atoms, {} in this case".
+                                 format(self.__trr_numatoms))
+            if numpy.max(sub) >= self.__trr_numatoms or numpy.min(sub) < 0:
+                raise IndexError("sub contains out of range elements for the given trajectory")
+            # sub appears to be valid
+            self.__sub = sub
+
+            # make tmp buffers
+            # C floats and C-order for arrays (see libxdrfile.i)
+            DIM = libxdrfile.DIM    # compiled-in dimension (most likely 3)
+            # both xdr and trr have positions
+            self.__pos_buf = numpy.zeros((self.__trr_numatoms, DIM), dtype=numpy.float32, order='C')
+            if self.format == "TRR":
+                # only trr has velocities and forces
+                self.__velocities_buf = numpy.zeros((self.__trr_numatoms, DIM), dtype=numpy.float32, order='C')
+                self.__forces_buf = numpy.zeros((self.__trr_numatoms, DIM), dtype=numpy.float32, order='C')
+            else:
+                # xdr does not have vel or forces.
+                self.__velocities_buf = None
+                self.__forces_buf = None
+        else:
+            self.__sub = None
+            self.__pos_buf = None
+            self.__velocities_buf = None
+            self.__forces_buf = None
+
+        # make the timestep, this is ALWAYS the used the public number of atoms
+        # (same as the calling Universe)
+        # at this time, __trr_numatoms and __sub are set, so self.numatoms has all it needs
+        # to determine number of atoms.
         self.ts = self._Timestep(self.numatoms)
+
         # Read in the first timestep
         self._read_next_timestep()
 
     @property
     def numatoms(self):
-        """Read the number of atoms from the trajectory.
+        """The number of publically available atoms that this reader will store in the timestep.
 
-        The result is cached. If for any reason the trajectory cannot
-        be read then 0 is returned.
+        If 'sub' was not given in the ctor, then this value will just be the actual
+        number of atoms in the underlying trajectory file. If however 'sub' was given, then
+        this value is the number specified by the 'sub' sub-selection.
+
+        If for any reason the trajectory cannot be read then a negative value is returned.
         """
-        if not self.__numatoms is None:   # return cached value
-            return self.__numatoms
-        try:
-            self.__numatoms = self._read_trj_natoms(self.filename)
-        except IOError:
-            return 0
-        else:
-            return self.__numatoms
+        return len(self.__sub) if self.__sub is not None else self.__trr_numatoms
 
     @property
     def numframes(self):
@@ -324,8 +421,9 @@ class TrjReader(base.Reader):
         if not self.__numframes is None:   # return cached value
             return self.__numframes
         try:
-            self.__numframes = self._read_trj_numframes(self.filename)
+            self._read_trj_numframes(self.filename)
         except IOError:
+            self.__numframes = 0
             return 0
         else:
             return self.__numframes
@@ -354,22 +452,35 @@ class TrjReader(base.Reader):
     def _read_trj_natoms(self, filename):
         """Generic number-of-atoms extractor with minimum intelligence. Override if necessary."""
         if self.format == 'XTC':
-            numatoms = libxdrfile.read_xtc_natoms(self.filename)
+            numatoms = libxdrfile.read_xtc_natoms(filename)
         elif self.format == 'TRR':
-            numatoms = libxdrfile.read_trr_natoms(self.filename)
+            numatoms = libxdrfile.read_trr_natoms(filename)
         else:
             raise NotImplementedError("Gromacs trajectory format %s not known." % self.format)
         return numatoms
 
     def _read_trj_numframes(self, filename):
-        """Generic numer-of-frames extractor with minimum intelligence. Override if necessary."""
+        """Number-of-frames extractor/indexer for XTC and TRR."""
         if self.format == 'XTC':
-            numframes = libxdrfile.read_xtc_numframes(self.filename)
+            self.__numframes, self.__offsets = libxdrfile.read_xtc_numframes(filename)
         elif self.format == 'TRR':
-            numframes = libxdrfile.read_trr_numframes(self.filename)
+            self.__numframes, self.__offsets = libxdrfile.read_trr_numframes(filename)
         else:
             raise NotImplementedError("Gromacs trajectory format %s not known." % self.format)
-        return numframes
+        return
+
+    def save_offsets(self, filename):
+        """Saves current trajectory offsets into filename (numpy format)."""
+        if self.__offsets is None:
+            self._read_trj_numframes(self.filename)
+        with open(filename, 'w') as OFILE:
+            numpy.save(OFILE, self.__offsets)
+
+    def load_offsets(self, filename):
+        """Loads current trajectory offsets from filename (numpy format)."""
+        with open(filename, 'r') as OFILE:
+            self.__offsets = numpy.load(OFILE)
+        self.__numframes = len(self.__offsets)
 
     def open_trajectory(self):
         """Open xdr trajectory file.
@@ -390,7 +501,7 @@ class TrjReader(base.Reader):
         ts.frame = 0
         ts.step = 0
         ts.time = 0
-        # additional data for xtc
+        # additional data for XTC
         ts.prec = 0
         # additional data for TRR
         ts.lmbda = 0
@@ -416,7 +527,7 @@ class TrjReader(base.Reader):
           *numatoms*
               number of atoms
           *delta*
-              Time interval between frames in ps.
+              Time interval between frames.
           *precision*
               accuracy for lossy XTC format as a power of 10 (ignored
               for TRR) [1000.0]
@@ -424,9 +535,12 @@ class TrjReader(base.Reader):
         :Returns: appropriate :class:`TrjWriter`
         """
         numatoms = kwargs.pop('numatoms', self.numatoms)
-        kwargs.pop('start', None)            # ignored by TrjWriter
-        kwargs['step'] = self.skip_timestep  # ignored/fixed to 1
+        kwargs['step'] = self.skip_timestep 
         kwargs.setdefault('delta', self.delta)
+        try:
+            kwargs['start'] = self[0].time/kwargs['delta']
+        except (AttributeError, ZeroDivisionError):
+            kwargs['start'] = 0
         try:
             kwargs.setdefault('precision', self.precision)
         except AttributeError:
@@ -459,12 +573,24 @@ class TrjReader(base.Reader):
             self.open_trajectory()
 
         if self.format == 'XTC':
-            ts.status, ts.step, ts.time, ts.prec = libxdrfile.read_xtc(self.xdrfile, ts._unitcell, ts._pos)
+            if self.__sub is None:
+                ts.status, ts.step, ts.time, ts.prec = libxdrfile.read_xtc(self.xdrfile, ts._unitcell, ts._pos)
+            else:
+                ts.status, ts.step, ts.time, ts.prec = libxdrfile.read_xtc(self.xdrfile, ts._unitcell, self.__pos_buf)
+                ts._pos[:] = self.__pos_buf[self.__sub]
         elif self.format == 'TRR':
-            ts.status, ts.step, ts.time, ts.lmbda = libxdrfile.read_trr(self.xdrfile, ts._unitcell, ts._pos,
-                                                                        ts._velocities, ts._forces)
+            if self.__sub is None:
+                ts.status, ts.step, ts.time, ts.lmbda, has_x, has_v, has_f = libxdrfile.read_trr(self.xdrfile, ts._unitcell, ts._pos,
+                                                                            ts._velocities, ts._forces)
+            else:
+                ts.status, ts.step, ts.time, ts.lmbda, has_x, has_v, has_f  = libxdrfile.read_trr(self.xdrfile, ts._unitcell, self.__pos_buf,
+                                                                            self.__velocities_buf, self.__forces_buf)
+                ts._pos[:] = self.__pos_buf[self.__sub]
+                ts._velocities[:] = self.__velocities_buf[self.__sub]
+                ts._forces[:] = self.__forces_buf[self.__sub]
         else:
             raise NotImplementedError("Gromacs trajectory format %s not known." % self.format)
+
         if (ts.status == libxdrfile.exdrENDOFFILE) or \
                 (ts.status == libxdrfile.exdrINT and self.format == 'TRR'):
             # seems that trr files can get a exdrINT when reaching EOF (??)
@@ -474,11 +600,16 @@ class TrjReader(base.Reader):
             raise IOError(errno.EFAULT, "Problem with %s file, status %s" %
                           (self.format, statno.errorcode[ts.status]), self.filename)
         if self.convert_units:
-            self.convert_pos_from_native(ts._pos)             # in-place !
+            # TRRs have the annoying possibility of frames without coordinates/velocities/forces...
+            if self.format == 'XTC' or has_x:
+                self.convert_pos_from_native(ts._pos)         # in-place !
             self.convert_pos_from_native(ts._unitcell)        # in-place ! (note: xtc/trr contain unit vecs!)
             ts.time = self.convert_time_from_native(ts.time)  # in-place does not work with scalars
             if self.format == 'TRR':
-                self.convert_velocities_from_native(ts._velocities) # in-place
+                if has_v:
+                    self.convert_velocities_from_native(ts._velocities) # in-place
+                if has_f:
+                    self.convert_forces_from_native(ts._forces)     # in-place
         ts.frame += 1
         return ts
 
@@ -491,23 +622,19 @@ class TrjReader(base.Reader):
         self.close()
         self.open_trajectory()
 
-    def _forward_to_frame(self, frameindex):
-        """Slow implementation: must read sequentially.
-
-        .. Note::
-
-           *frameindex* starts from 0; i.e. *frameindex* = frame - 1.
+    def _goto_frame(self, frame):
+        """ Fast, index-based, random frame access
 
         :TODO: Should we treat frame as 0-based or 1-based??  Right
                now: 0-based (which is inconsistent) but analogous to
                DCDReader
         """
-        self.rewind()
-        if frameindex == 0:
-            return
-        for ts in self:
-            if ts.frame-1 >= frameindex:
-                break
+        if self.__offsets is None:
+            self._read_trj_numframes(self.filename)
+        self._seek(self.__offsets[frame])
+        self._read_next_timestep()
+        self.ts.frame = frame+1
+        return self.ts
 
     def __getitem__(self, frame):
         if (numpy.dtype(type(frame)) != numpy.dtype(int)) and (type(frame) != slice):
@@ -518,25 +645,45 @@ class TrjReader(base.Reader):
                 frame = len(self) + frame
             if (frame < 0) or (frame >= len(self)):
                 raise IndexError("0 <= frame < len(traj) is outside of trajectory boundaries")
-            self._forward_to_frame(frame)
-            return self.ts
+            return self._goto_frame(frame)
         elif type(frame) == slice: # if frame is a slice object
             if not (((type(frame.start) == int) or (frame.start == None)) and
                     ((type(frame.stop) == int) or (frame.stop == None)) and
                     ((type(frame.step) == int) or (frame.step == None))):
                 raise TypeError("Slice indices are not integers")
-            def iterDCD(start=frame.start, stop=frame.stop, step=frame.step):
+            def _iter(start=frame.start, stop=frame.stop, step=frame.step):
+                # check_slices implicitly calls len(self), thus setting numframes
                 start, stop, step = self._check_slice_indices(start, stop, step)
-                if step < 0:
-                    raise NotImplementedError("XTC/TRR do not support reverse iterating for performance reasons")
-                for ts in self:
-                    # simple sequential plodding through trajectory --- SLOW!
-                    frameindex = ts.frame - 1
-                    if frameindex < start:  continue
-                    if frameindex >= stop:  break
-                    if (frameindex - start) % step != 0: continue
-                    yield self.ts
-            return iterDCD()
+                if step == 1:
+                    rng = xrange(start+1,stop,step)
+                    yield self._goto_frame(start)
+                    for framenr in rng:
+                        yield self._read_next_timestep()
+                else:
+                    rng = xrange(start,stop,step)
+                    for framenr in rng:
+                        yield self._goto_frame(framenr)
+            return _iter()
+
+    def _check_slice_indices(self, start, stop, step):
+        if step == None: step = 1
+        if (start < 0) and start is not None: start += len(self)
+        if (stop < 0) and stop is not None: stop += len(self)
+        if step > 0:
+            if start == None: start = 0
+            if stop == None: stop = len(self)
+        else:  # Fixes reverse iteration with default slice. (eg. trajectory[::-1])
+            if start == None: start  = len(self)-1
+            if  stop == None: stop   = -1
+        if stop > len(self): stop = len(self)
+
+        if step > 0 and stop <= start:
+            raise IndexError("Stop frame is lower than start frame")
+        if step == 0:
+            raise IndexError("Iteration step 0.")
+        if start >= len(self):
+            raise IndexError("Frame start outside of the range of the trajectory.")
+        return (start, stop, step)
 
     def timeseries(self, asel, start=0, stop=-1, skip=1, format='afc'):
         raise NotImplementedError("timeseries not available for Gromacs trajectories")
@@ -545,3 +692,24 @@ class TrjReader(base.Reader):
 
     def __del__(self):
         self.close()
+
+    def _seek(self, pos, rel=False):
+        """Traj seeker"""
+        if self.format == 'XTC' or self.format == 'TRR':
+            if rel:
+                status = libxdrfile.xdr_seek(self.xdrfile, long(pos), libxdrfile.SEEK_CUR)
+            else:
+                status = libxdrfile.xdr_seek(self.xdrfile, long(pos), libxdrfile.SEEK_SET)
+            if status != libxdrfile.exdrOK:
+                raise IOError(errno.EIO, "Problem seeking to offset %d (relative to current = %s) on file %s, status %s" % (pos, rel, self.filename, status))
+        else:
+            raise NotImplementedError("Gromacs trajectory format %s not known." % self.format)
+
+    def _tell(self):
+        """Traj pos getter"""
+        if self.format == 'XTC' or self.format == 'TRR':
+            offset = libxdrfile.xdr_tell(self.xdrfile)
+        else:
+            raise NotImplementedError("Gromacs trajectory format %s not known." % self.format)
+        return offset
+
